require 'magic-resource/version'

require 'ruby-features'
require 'activerecord-devkit'

require 'magic-resource/railtie'

module MagicResource
  autoload :Controller,         'magic-resource/controller'
  autoload :Helper,             'magic-resource/helper'
  autoload :Container,          'magic-resource/container'
  autoload :PathWithHttpVerb,   'magic-resource/path_with_http_verb'

  module Generators
    autoload :InstallGenerator, 'generators/magic_resource/install_generator'
  end

  MAGIC_PARAMETERS = %w(
    force_resource_name_definition
    assert_resource_name_in_controller
    disable_content_for_helper
  ).freeze
  MAGIC_VALID_VALUES = [false, :by_exception, :by_warning].freeze

  mattr_accessor :default_context
  @@default_context = :crud

  mattr_accessor :default_form_type
  @@default_form_type = :simple

  MAGIC_PARAMETERS.each do |param|
    mattr_reader param.to_sym
    class_variable_set(:"@@#{param}", :by_exception)

    define_singleton_method(:"#{param}=") do |value|
      raise ArgumentError.new(
        "Unknown value: #{value.inspect}. Valid values are: #{MAGIC_VALID_VALUES.inspect}"
      ) unless MAGIC_VALID_VALUES.include?(value)
      class_variable_set(:"@@#{param}", value)
    end
  end

  def self.setup
    yield self
  end

  def self.no_magic
    set_all_magic_parameters(:by_exception)
  end

  def self.try_magic?
    set_all_magic_parameters(:by_warning)
  end

  def self.full_magic!
    set_all_magic_parameters(false)
  end

  def self.set_all_magic_parameters(value)
    MAGIC_PARAMETERS.each do |param|
      public_send(:"#{param}=", value)
    end
  end

  def self.logger
    ::Rails.logger
  end

  def self.assert_restriction(restriction_type, message)
    message.prepend('MagicResource: ')
    case class_variable_get(:"@@#{restriction_type}")
    when :by_exception then raise(message)
    when :by_warning then logger.warn(message)
    end
  end
end
