module MagicResource
  module Controller
    extend ActiveSupport::Concern

    included do
      helper MagicResource::Helper
      helper_method :resource_container, :with_resource_container, :resource_contents

      prepend_before_action :prepare_resource_controller

      protected
      attr_reader :resources, :resource

      private
      attr_accessor :resource_name, :resource_context, :resource_container
    end

    protected

    def resources=(resources)
      @resources, @resource = resources, nil
      prepare_resource_container(resources)
    end

    def resource=(resource)
      @resources, @resource = nil, resource
      prepare_resource_container(resource)
    end

    def r(*args)
      MagicResource::Container._launch(self, *args)
    end

    private

    def prepare_resource_controller
      resource_config = self.class.name.match(/^(?<name>\w+?)::(?<context>\w+)Controller$/)
      raise "Wrong resource controller name: #{self.class.name}" unless resource_config

      self.resource_name = resource_config[:name].underscore.to_sym
      self.resource_context = resource_config[:context].underscore.to_sym

      prepare_resource_container
    end

    def prepare_resource_container(rs = nil)
      _prepare_resource_container(rs)

      MagicResource.assert_restriction(
        :assert_resource_name_in_controller,
        "Wrong resource name in controller, expected #{resource_name.inspect} but got #{resource_container.name.inspect}"
      ) unless resource_container.name == resource_name
    end

    def _prepare_resource_container(rs)
      self.resource_container = MagicResource::Container.new(rs || resource_name, resource_context)
    end

    def with_resource_container(new_resource_container)
      same_resource_container = resource_container == new_resource_container
      old_resource_container, self.resource_container = resource_container, new_resource_container unless same_resource_container

      yield
    ensure
      self.resource_container = old_resource_container unless same_resource_container
    end

    def resource_contents
      @_resource_contents ||= {}
    end

  end
end
